--[[
  一般的な DLNA クライアント用スクリプト
]]


-- この機種がサポートしているメディア情報のリスト
SUPPORT_MEDIA_LIST = {
  "video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AAC_MULT5",
  "video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AC3",
  "video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_MPEG1_L3",
  "video/mp4:*",
  "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC",
  "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC_XAC3",
  "video/mpeg:*",
  "video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_JP_AAC_T",
  "video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_JP_T",
}


--[[
 GetMimeType 関数

 メディアファイルに応じたメディア情報を返す。

 引数：
   fname: ファイル名
   minfo: MediaInfo.dll などから得られた情報
]]
function GetMimeType(fname, minfo)
  local ext = string.match(fname, "%.([^.]-)$")
  if ext then
    ext = string.lower(ext)
    
    if ext == "mp4" or ext == "flv" or ext == "mkv" or ext == "avi" then
      if minfo.Video.Format == "AVC" then
        -- remux to m2ts
        return "video/vnd.dlna.mpeg-tts"
          ..":DLNA.ORG_PN=AVC_TS_JP_AAC_T"
          ..";DLNA.ORG_OP=10"
          ..":::TRANS:::"
      else
        return "video/mpeg"
          ..":DLNA.ORG_PN=MPEG_PS_NTSC"
          ..";DLNA.ORG_OP=10"
          ..":::TRANS:::"
      end
    end
    
    if ext == "m2ts" then
      if minfo.Video.Format == "AVC" then
        return "video/vnd.dlna.mpeg-tts"
          ..":DLNA.ORG_PN=AVC_TS_JP_AAC_T"
          ..";DLNA.ORG_OP=10"
      elseif minfo.Video.Format == "MPEG Video" then
        return "video/vnd.dlna.mpeg-tts"
          ..":DLNA.ORG_PN=MPEG_TS_JP_T"
          ..";DLNA.ORG_OP=10"
      end
    end
    
    if ext == "mpg" or ext == "mpeg" then
      return "video/mpeg"
        ..":DLNA.ORG_PN=MPEG_PS_NTSC"
        ..";DLNA.ORG_OP=10"
    end
    
    if ext == "iso" then
      return "video/mpeg"
        ..":DLNA.ORG_PN=MPEG_PS_NTSC"
        ..";DLNA.ORG_OP=10"
        ..":::TRANS:::"
    end
    
    if ext == "mp3" or ext == "wav" or ext == "avs" then
      return "video/mpeg"
        ..":DLNA.ORG_PN=MPEG_PS_NTSC"
        ..";DLNA.ORG_OP=10"
        ..":::TRANS:::"
    end
  end
  return ""
end


--[[
 GetTranscodeCommand 関数
 
 トランスコードのコマンドを返す。

 引数：
   fname: ファイル名
   minfo: MediaInfo.dll などから得られた情報
]]
function GetTranscodeCommand(fname, minfo)
  local ext = string.match(fname, "%.([^.]-)$")
  if ext then
    ext = string.lower(ext)
    if ext == "mp4" or ext == "flv" or ext == "mkv" or ext == "avi" then
      if minfo.Video.Format == "AVC" then
        if minfo["Audio"]["Codec/String"] == "AAC LC" or minfo["Audio"]["Codec/String"] == "AC3" or minfo["Audio"]["Codec/String"] == "MPEG-1 Audio layer 3" then
          return [[]]
          ..[[
            [(M2TS) $_name_$]
            ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
            -vcodec copy -vbsf h264_mp4toannexb
            -acodec copy
            -f mpegts -mpegts_m2ts_mode 1
            -y "$_out_$"
          ]]
        else
          return [[]]
          ..[[
            [(M2TS AC-3) $_name_$]
            ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
            -vcodec copy -vbsf h264_mp4toannexb
            -acodec ac3 -ar 48000 -ab 192k
            -f mpegts -mpegts_m2ts_mode 1
            -y "$_out_$"
          ]]
          ..[[
            [(M2TS AAC) $_name_$]
            ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
            -vcodec copy -vbsf h264_mp4toannexb
            -acodec libfaac -ar 48000 -ab 192k
            -f mpegts -mpegts_m2ts_mode 1
            -y "$_out_$"
          ]]
        end
      else
        return [[]]
        ..[[
          [(MPEG2-PS) $_name_$]
          ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
          -vcodec mpeg2video -r 30000/1001 -b 5000k
          -acodec ac3 -ar 48000 -ab 192k
          -f dvd
          -y "$_out_$"
        ]]
      end
    end
    
    -- viera cannot play audio only
    -- audio to video for viera
    if ext == "mp3" or ext == "wav" then
      if minfo["Audio"]["Codec/String"] == "MPEG-1 Audio layer 3" then
        return [[]]
        ..[[
          [(MPEG2-PS nullsrc) $_name_$]
          ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
          -f lavfi -i nullsrc="size=640x360:rate=30000/1001" -shortest
          -vcodec mpeg2video -r 30000/1001
          -acodec copy
          -f dvd
          -y "$_out_$"
        ]]
        ..[[
          [(MPEG2-PS drawtext) $_name_$]
          ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
          -f lavfi -i nullsrc="size=1280x720:rate=30000/1001" -shortest
          -vf drawtext=fontfile=MTLc3m.ttf:text="]]..minfo.General.FileName..[[":fontsize=48:x="(2/PI*asin(sin(2*PI*(n*2/w))))*(w-text_w)/2+(w-text_w)/2":y="(2/PI*asin(sin(2*PI*(n*2/h))))*(h-text_h)/2+(h-text_h)/2"
          -vcodec mpeg2video -r 30000/1001 -b 5000k
          -acodec copy
          -f dvd
          -y "$_out_$"
        ]]
        ..[[
          [(MPEG2-PS all play noseek buggy) $_name_$]
          cmd /s /c "type "]]..minfo.General.FolderName..[[\*.]]..minfo.General.FileExtension..[[" | 
          ffmpeg $_ff_ss_$ $_ff_t_$ -i -
          -f lavfi -i nullsrc="size=640x360:rate=30000/1001" -shortest
          -vcodec mpeg2video -r 30000/1001
          -acodec copy
          -f dvd
          -y "$_out_$""
        ]]
      else
        return [[]]
        ..[[
          [(MPEG2-PS nullsrc) $_name_$]
          ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
          -f lavfi -i nullsrc="size=640x360:rate=30000/1001" -shortest
          -vcodec mpeg2video -r 30000/1001
          -acodec ac3 -ar 48000 -ab 192k
          -f dvd
          -y "$_out_$"
        ]]
        ..[[
          [(MPEG2-PS drawtext) $_name_$]
          ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
          -f lavfi -i nullsrc="size=1280x720:rate=30000/1001" -shortest
          -vf drawtext=fontfile=MTLc3m.ttf:text="]]..minfo.General.FileName..[[":fontsize=48:x="(2/PI*asin(sin(2*PI*(n*2/w))))*(w-text_w)/2+(w-text_w)/2":y="(2/PI*asin(sin(2*PI*(n*2/h))))*(h-text_h)/2+(h-text_h)/2"
          -vcodec mpeg2video -r 30000/1001 -b 5000k
          -acodec ac3 -ar 48000 -ab 192k
          -f dvd
          -y "$_out_$"
        ]]
      end
    end
    
    if ext == "avs" then
      return [[]]
      ..[[
        [(MPEG2-PS) $_name_$]
        ffmpeg $_ff_ss_$ $_ff_t_$ -i "$_in_$"
        -vcodec mpeg2video -r 30000/1001 -b 5000k
        -acodec ac3 -ar 48000 -ab 192k
        -f dvd
        -y "$_out_$"
      ]]
    end
    
    if ext == "iso" and minfo.General.Format == "ISO DVD" then
      if minfo.DVD.ALANG.en and minfo.DVD.SLANG.ja then
        -- 英語音声と日本語字幕がある場合
        return [[]]
        ..[[
          [(英語音声・日本語字幕) $_name_$]
          mencoder -quiet $_me_ss_sb_$ $_me_endpos_$ -dvd-device "$_in_$" dvd://$_longest_$ -o "$_out_$"
          -ovc lavc -ofps 30000/1001 -vf pp=l5,harddup -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0
          -oac lavc -channels 2 -srate 48000 -af lavcresample=48000 -lavcopts acodec=ac3:abitrate=192
          -alang en -slang ja
          -of mpeg -mpegopts format=dvd:tsaf
        ]]
        ..[[
          [(日本語音声) $_name_$]
          mencoder -quiet $_me_ss_sb_$ $_me_endpos_$ -dvd-device "$_in_$" dvd://$_longest_$ -o "$_out_$"
          -ovc lavc -ofps 30000/1001 -vf pp=l5,harddup -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0
          -oac lavc -channels 2 -srate 48000 -af lavcresample=48000 -lavcopts acodec=ac3:abitrate=192
          -alang ja -slang ja -forcedsubsonly
          -of mpeg -mpegopts format=dvd:tsaf
        ]]
        ..[[
          [(英語音声) $_name_$]
          mencoder -quiet $_me_ss_sb_$ $_me_endpos_$ -dvd-device "$_in_$" dvd://$_longest_$ -o "$_out_$"
          -ovc lavc -ofps 30000/1001 -vf pp=l5,harddup -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0
          -oac lavc -channels 2 -srate 48000 -af lavcresample=48000 -lavcopts acodec=ac3:abitrate=192
          -alang en -slang en -forcedsubsonly
          -of mpeg -mpegopts format=dvd:tsaf
        ]]
      else
        return [[]]
        ..[[
          [$_name_$]
          mencoder -quiet $_me_ss_sb_$ $_me_endpos_$ -dvd-device "$_in_$" dvd://$_longest_$ -o "$_out_$"
          -ovc lavc -ofps 30000/1001 -vf pp=l5,harddup -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0
          -oac lavc -channels 2 -srate 48000 -af lavcresample=48000 -lavcopts acodec=ac3:abitrate=192
          -forcedsubsonly
          -of mpeg -mpegopts format=dvd:tsaf
        ]]
      end
    end
  end
end
